; POLYGON4.CNC
;
; Macro to pocket out interior of a regular polygon
;
;   Marc Leonard, CNC Services Northwest, 12/21/2004
;
; Call: G65 "POLYGON4.CNC" X_ Y_ Z_ W_ A_ H_ U_ C_ D_ F_
;
; Arguments:
;
;   X, Y  Center point
;   Z     Cut depth
;   W     Clearance plane (Z)
;   A     Number of sides (3 or more)
;   H     Height (across flats if A is even; point-to-flat if A is odd)
;   U     Stepover
;   C     Direction (1 for CCW, 0 for CW)
;   D     Angle from center to middle of first flat
;   F     Cutting feedrate

 IF #A < 3 || #A > 1000 THEN ERROR

 #108 = 1.0
 IF #C == 0 THEN #108 = -1.0  ; direction of angle progression
 #109 = 360.0 / #A            ; exterior angle
 #110 = 180.0 - #109          ; interior angle

 #120 = #U * 2.0
 IF [#A % 2] != 0 THEN #120 = #U * [1.0 + 1.0/SIN[#110/2.0]]

 ; Loop to calculate factor for length of a side:
 #101 = 0.0      ; angle accumulator
 #102 = 0.0      ; sum of sines
 #100 = #A / 2   ; loop counter
 N1
  #101 = #101 + #109
  #102 = #102 + SIN[#101]
  #100 = #100 - 1
 IF #100 > 0.75 THEN GOTO 1

 #121 = 0.0
 #122 = #D

 ;---- Rapid over center, then feed to depth ----
 G0 X#X Y#Y Z#W
 G1 F[#F * 0.5] Z#Z
 ;----

 N2
  #121 = #121 + #120
  IF #121 > #H THEN #121 = #H

  #112 = #121 / #102     ; length of a side

  #111 = [#112 / 2.0] * TAN[#110 / 2.0] ; distance from center to flat

  #113 = #122 - #108 * #109 / 2.0  ; current direction of travel
  #114 = #X + #111 * COS[#113]   ; last X position
  #115 = #Y + #111 * SIN[#113]   ; last Y position

  ;---- Move out to start of arc ----
  G1 X#114 Y#115 F[#F * 0.5]
  ;----

  #114 = #X + #111 * COS[#122]
  #115 = #Y + #111 * SIN[#122]

  ;---- Arc to middle of flat side ----
  G[2+#C] X#114 Y#115 R#111 F#F
  ;----

  #113 = #122 + #108 * 90.0
  #114 = #114 + #112 * COS[#113] / 2.0
  #115 = #115 + #112 * SIN[#113] / 2.0

  ;---- Move remaining half way along side to first vertex ----
  G1 X#114 Y#115 F#F
  ;----

  #100 = #A - 1   ; loop counter: loop around n-1 full sides

  N3
   #113 = #113 + #108 * #109
   #114 = #114 + #112 * COS[#113]
   #115 = #115 + #112 * SIN[#113]

   ;---- Move to next vertex ----
   G1 X#114 Y#115 F#F
   ;----

   #100 = #100 - 1
  IF #100 > 0 THEN GOTO 3

  #113 = #113 + #108 * #109

  #114 = #114 + #112 * COS[#113] / 2.0
  #115 = #115 + #112 * SIN[#113] / 2.0

  ;---- Move half way along last (first) side ----
  G1 X#114 Y#115 F#F
  ;----

  #113 = #122 + #108 * #109 / 2.0  ; current direction of travel
  #114 = #X + #111 * COS[#113]   ; last X position
  #115 = #Y + #111 * SIN[#113]   ; last Y position

  IF #121 < #H THEN GOTO 4
   ;---- Arc away from wall, only on final pass ----
   G[2+#C] X#114 Y#115 R#111
   ;----
  N4

  #122 = #122 + #108 * #109

 IF #121 < #H THEN GOTO 2

 ;---- Rapid back up to clearance ----
 G0 Z#W
 ;----

M99
